<div class="modal-data-viewer-embed-dialog modal fade in">
  <div class="modal-header">
    <a class="close" data-dismiss="modal">×</a>
    <h3>{{ _('Embed Data Viewer') }}</h3>
  </div>
  <div class="modal-body">
    <div>
      <p>{{ _('Embed this view by copying this into your webpage:') }}</p>
      <textarea class="embedIframeText" style="width: 100%; height: 200px;"></textarea>
      <p>{{ _('Choose width and height in pixels:') }}</p>
      <label for="iframe-width">{{ _('Width:') }}</label>
      <input class="iframe-width" name="iframe-width" value="800"/>
      <label for="iframe-height">{{ _('Height:') }}</label>
      <input class="iframe-height" name="iframe-height" value="500"/>
    </div>
    <a class="embedLink" href="">{{ _('Preview') }}</a>
  </div>
</div>
