<li>{% link_for _('Datasets'), controller='package', action='search', highlight_actions = 'new index' %}</li>
<li class="active">{% link_for _('Create Dataset'), controller='package', action='new' %}</li>
