.search-form {
  // .clearfix;
  margin-bottom: 20px;
  padding-bottom: 25px;
  border-bottom: 1px dotted @genericBorderColor;

  // Normal search box
  .search-input {
    position: relative;
    margin-bottom: 20px;
  	input {
      .box-sizing(border-box);
      margin: 0;
      width: 100%;
      height: auto;
    }
    button {
      cursor: pointer;
      display: block;
      position: absolute;
      top: 50%;
      margin-top: -10px;
      right: 10px;
      height: 20px;
      padding: 0;
      border: none;
      background: transparent;
      span {
        display: none;
      }
      i {
        color: @inputBorder;
        .transition(color 0.2s ease-in);
      }
      &:hover i {
        color: @inputColor;
      }
    }
    &.search-giant {
      input {
        font-size: 16px;
        padding: 15px;
      }
      button {
        margin-top: -4px;
        right: 15px;
        height: 30px;
        i {
          font-size: 28px;
          width: 28px;
        }
      }
    }
  }
  .control-order-by {
    label,
    select {
      display: inline;
    }
    select {
      width: 160px;
      margin: 0;
    }
  }
  h2 {
    font-size: 24px;
    line-height: 1.3;
    color: @layoutBoldColor;
    margin-bottom: 0;
    margin-top:20px;
  }
  .filter-list {
    color: @layoutTextColor;
    line-height: 32px;
    margin: 10px 0 0 0;
    .pill {
      line-height: 21px;
    }
    .extra {
      margin-top: 10px;
      font-size: 18px;
      font-weight: normal;
      color: @layoutBoldColor;
    }
  }
  &.no-bottom-border {
    border-bottom-width: 0;
    margin-bottom: 0;
  }
}

.tertiary {
  .control-order-by {
    float: none;
    margin: 0;
    label {
      display: block;
      margin-bottom: 5px;
      font-weight: normal;
      font-size: 12px;
    }
    select {
      display: block;
      font-size: 12px;
      width: 100%;
    }
  }
  .search-input {
    margin-bottom: 10px;
  }
}
@media (min-width: 980px) {
  .search-form .control-order-by {
    float:right;
    margin-left:15px;
  }
  .tertiary {
    .search-form .control-order-by {
      float: none;
      margin: 0;
    }
  }
}
