{% extends "revision/read_base.html" %}

{% block subtitle %}{{ _('Revision History') }}{% endblock %}

{% block breadcrumb_content %}
  <li class="active"><a href="" title="{{ _('Revision History') }}">{{ _('Revisions') }}</a></li>
{% endblock %}

{% block primary_content_inner %}
  <h1>{{ _('Revision History') }}</h1>

  {{ c.page.pager() }}

  {% block revisions_list %}
    {% snippet "revision/snippets/revisions_list.html", revisions=c.page.items %}
  {% endblock %}

  {{ c.page.pager() }}
{% endblock %}
