{% extends "page.html" %}

{% block subtitle %}
  {{ _('Manage') }} - {{ h.get_translated(c.group_dict, 'title') or c.group_dict.display_name }} - {{ _('Groups') }}
{% endblock %}

{% set group = c.group_dict %}

{% block breadcrumb_content %}
  <li>{% link_for _('Groups'), controller='group', action='index' %}</li>
  {% block breadcrumb_content_inner %}
    <li>{% link_for h.get_translated(group, 'title') or group.display_name |truncate(35), controller='group', action='read', id=group.name %}</li>
    <li class="active">{% link_for _('Manage'), controller='group', action='edit', id=group.name %}</li>
  {% endblock %}
{% endblock %}

{% block content_action %}
  {% link_for _('View'), controller='group', action='read', id=c.group_dict.name, class_='btn', icon='eye' %}
{% endblock %}

{% block content_primary_nav %}
  {{ h.build_nav_icon('group_edit', _('Edit'), id=c.group_dict.name) }}
  {{ h.build_nav_icon('group_members', _('Members'), id=c.group_dict.name) }}
{% endblock %}

{% block secondary_content %}
  {% snippet "group/snippets/info.html", group=c.group_dict, show_nums=false %}
{% endblock %}
