{% extends "organization/base_form_page.html" %}

{% block subtitle %}{{ _('Create an Organization') }}{% endblock %}

{% block breadcrumb_link %}{{ h.nav_link(_('Create an Organization'), controller='organization', action='edit', id=c.organization.name) }}{% endblock %}

{% block page_heading %}{{ _('Create an Organization') }}{% endblock %}

{% block page_header %}{% endblock %}

{% block breadcrumb_content_inner %}
  <li class="active"><a href="#">{{ _('Create an Organization') }}</a></li>
{% endblock %}

{% block secondary_content %}
  {% snippet "organization/snippets/helper.html" %}
{% endblock %}
