{% extends "organization/edit_base.html" %}

{% block subtitle %}{{ _('Members') }} - {{ super() }}{% endblock %}

{% block page_primary_action %}
  {% if h.check_access('organization_update', {'id': organization.id}) %}

    {% link_for _('Add Member'), controller='organization', action='member_new', id=c.group_dict.id, class_='btn btn-primary', icon='plus-square' %}
  {% endif %}
{% endblock %}

{% block primary_content_inner %}
  {% set count = c.members|length %}
  {% set members_count = ungettext('{count} member', '{count} members', count).format(count=count) %}
  <h3 class="page-heading">{{ members_count }}</h3>
  <table class="table table-header table-hover table-bordered">
    <thead>
      <tr>
          <th>{{ _('User') }}</th>
          <th>{{ _('Role') }}</th>
          <th></th>
      </tr>
    </thead>
    <tbody>
      {% for user_id, user, role in c.members %}
        <tr>
          <td class="media">
            {{ h.linked_user(user_id, maxlength=20) }}
          </td>
          <td>{{ role }}</td>
          <td>
            <div class="btn-group pull-right">
              <a class="btn btn-small" href="{% url_for controller='organization', action='member_new', id=c.group_dict.id, user=user_id %}" title="{{ _('Edit') }}">
                <i class="fa fa-wrench"></i>
              </a>
              <a class="btn btn-danger btn-small" href="{% url_for controller='organization', action='member_delete', id=c.group_dict.id, user=user_id %}" data-module="confirm-action" data-module-content="{{ _('Are you sure you want to delete this member?') }}" title="{{ _('Delete') }}">{% block delete_button_text %}<i class="fa fa-times"></i>{% endblock %}</a>
            </div>
          </td>
        </tr>
      {% endfor %}
    </tbody>
  </table>
{% endblock %}
