{% extends "organization/base_form_page.html" %}

{% block subtitle %}{{ _('Edit') }} - {{ super() }}{% endblock %}

{% block page_heading_class %}hide-heading{% endblock %}
{% block page_heading %}{{ _('Edit Organization') }}{% endblock %}
